@@ -1,4 +1,3 @@
-#%%
 import pandas as pd
 import regex as re
 import numpy as np
@@ -28,42 +27,32 @@ logger.addHandler(file_handler)
 
 class QualityAssessments:
 
-    def __init__(self,main_error_tracker_name,tracer_error_tracker_name,naming_conv_tracker_name,util_object=None):
-        self.main_error_tracker_name = main_error_tracker_name
-        self.tracer_error_tracker_name = tracer_error_tracker_name
-        self.naming_conv_tracker_name = naming_conv_tracker_name
+    def __init__(self,util_object=None):
+
         if util_object != None:
             self.util = util_object
     
-    def null_values_checker(self,df,cols_to_group,cols_to_ignore,tab_name,
-                    null_definitions=[np.nan,'N/A','','None'],output_method='Gsheet'):
+    def null_values_checker(self,df,cols_to_group,cols_to_ignore,gsheet_name,tab_name,
+                    null_definitions=[np.nan,'N/A','','None'],output_method='gsheet'):
         """Takes in a dataframe and columns to groupby and checks how many null values or null equivalents
-            there are in the rest of the columns. 
+        there are in the rest of the columns.
             
-            Parameters
-            -------------------
-            df : DataFrame 
-                The Input Dataframe of any type where you want to check for nulls
-            cols_to_group : list, str
-                The list of columns to perform a groupby operation with the null percentage
-                counts will be a percentage of nulls in these groupbys
-            cols_to_ignore : list, str
-                The list of columns to not count nulls in, to make the output dataframe smaller perhaps
-            gsheet_name : str
-                The name of the google sheet workbook to pass to the google sheet function
-            tab_name : str
-                The name of the tab in the google sheet workbook to pass to the google sheet function
-                The google sheet must be setup with this tab already created
-            null_definitions : list
-                A list containing elements to be defined as a null value 
-            output_method : str
-                A string identifying whether the output is to be sent to a Google sheet ('Gsheet') or returned
-                as a dataframe ('Dataframe')
-            
-            Returns
-            -------------------
-            null_count_df : DataFrame
-                Dataframe showing the percentage of nulls in each column grouped by the 'cols_to_group'"""
+        Args:
+            df (pandas.DataFrame): The Input Dataframe of any type where you want to check for nulls.
+            cols_to_group (list of str): The list of columns to perform a groupby operation with the null 
+                                        percentage counts will be a percentage of nulls in these groupbys.
+            cols_to_ignore (list of str): The list of columns to not count nulls in, to make the output 
+                                        dataframe smaller perhaps.
+            gsheet_name (str): The name of the google sheet workbook to pass to the google sheet function.
+            tab_name (str): The name of the tab in the google sheet workbook to pass to the google sheet function.
+                                            The google sheet must be setup with this tab already created.
+            null_definitions (list): A list containing elements to be defined as a null value.
+            output_method (str): A string identifying whether the output is to be sent to a Google sheet 
+                                        ('gsheet') or returned as a dataframe ('Dataframe').
+                             
+        Returns:
+            null_count_df (pandas.DataFrame): Dataframe showing the percentage of nulls in each column grouped 
+                                           by the 'cols_to_group'."""
 
         for null in null_definitions: 
             df = df.replace(null,'NullValue')
@@ -75,45 +64,45 @@ class QualityAssessments:
             col_groupby = df.groupby(cols_to_group)[col].apply(lambda x: round(100 * x[x == 'NullValue'].count()/x.shape[0],1)).reset_index()
             null_count_df = pd.merge(null_count_df,col_groupby)
 
-        if output_method == 'Gsheet':
-            self.util.write_to_gsheet(self.main_error_tracker_name,tab_name,null_count_df)
+        if output_method == 'gsheet':
+            self.util.write_to_gsheet(gsheet_name,tab_name,null_count_df)
         elif output_method == 'Dataframe': 
             return null_count_df
     
-    def check_data_recency(self,df,cols_to_group,three_days_for_monday= True,date_col='date',gsheet_tab='DataRecency'):
+    def check_data_recency(self,df,cols_to_group,gsheet_name,tab_name='DataRecency',
+                            three_days_for_monday= True,date_col='date',
+                            dayfirst='EnterValue',yearfirst='EnterValue',format=None,errors='raise'):
         """Post a google sheet showing how many days since different channels have been active.
-            Also return a list of channels that have been inactive for more than 2 days which
-            might be indicative of an error
-            
-            Parameters
-            ------------------
-            df : DataFrame
-                Dataframe of data containing a 'date' column 
-            cols_to_group : list, str
-                Columns to groupby effectively creating the 'channels' 
-            three_days_for_monday : bool
-                If the check is run on a Monday give 3 days before declaring a
-                channel as inactive because of the weekend
-            date_col : str
-                Column name for the date to find the maximum value for based on grouping by 'cols_to_group'
-                If 'created' is used when working with Tracer data then this will find out when the data was 
-                last updated by Tracer, regardless of whether the actual date was of the post was 30 days ago
-            gsheet_tab : str
+        Also return a list of channels that have been inactive for more than 2 days which
+        might be indicative of an error
+        
+        Args:
+            df (pandas.DataFrame): Dataframe of data containing a 'date' column
+            cols_to_group (list or str): Columns to groupby effectively creating the 'channels'
+            gsheet_name (str): Name of the google sheet to write to
+            tab_name (str): Name of the tab in the Google sheet
+            three_days_for_monday (bool): If the check is run on a Monday, give 3 days before declaring a channel as inactive because of the weekend.
+            date_col (str): Column name for the date to find the maximum value for based on grouping by `cols_to_group`. If 'created' is used when working with Tracer data, then this will find out when the data was last updated by Tracer, regardless of whether the actual date of the post was 30 days ago.
+            dayfirst (bool): If True, parses dates with the day first, eg 10/11/12 is parsed as 2012-11-10. If False, parses dates with the month first, eg 10/11/12 is parsed as 2010-11-12. If None, this is set to True if the day is in the first position in the format string, False otherwise. If dayfirst is set to True, parsing will be faster, but will fail for ambiguous dates, such as 01/02/03.
+            yearfirst (bool): If True parses dates with the year first, eg 10/11/12 is parsed as 2010-11-12. If both dayfirst and yearfirst are True, yearfirst is preceded (same as dateutil). If False, parses dates with the month first, eg 10/11/12 is parsed as 2012-11-10. If None, this defaults to False. Setting yearfirst to True is not recommended, as it can result in ambiguous dates.
+            format (str): Format to use for strptime. If None, the format is inferred from the first non-NaN element of the column. If the format is inferred, it will be used in subsequent parsing, even if the format changes. To specify a format string that will be used in parsing regardless of the inferred format, use pd.to_datetime with format.
+            errors (str): If 'raise', then invalid parsing will raise an exception. If 'coerce', then invalid parsing will be set as NaT. If 'ignore', then invalid parsing will return the input.
 
-
-            Returns
-            ------------------
-            error_message : str
-                String error message describing which channels are recently inactive. This message can then be sent
-                to a slack function"""
+        Returns:
+            error_message (str): String error message describing which channels are recently inactive. This message can then be sent
+                                to a slack function"""
 
         error_message = ''
         organic_or_paid = self.util.identify_paid_or_organic(df)
         print(organic_or_paid)
         buffer_days_since_active = 2
-        data_recency = pd.DataFrame(df.groupby(cols_to_group)[date_col].max().apply(lambda x: (today - x).days)).reset_index().rename(columns={'date':'DaysSinceActive'})  
+        #remove any timezone information from the date_col and check the date is being read in in correct format
+        df[date_col] = pd.to_datetime(df[date_col].dt.date,dayfirst=dayfirst,
+                                      yearfirst=yearfirst,format=format,errors=errors)
+
+        data_recency = pd.DataFrame(df.groupby(cols_to_group)[date_col].max().apply(lambda x: (today - x).days)).reset_index().rename(columns={date_col:'DaysSinceActive'})  
     
-        self.util.write_to_gsheet(self.main_error_tracker_name,gsheet_tab,data_recency,sheet_prefix=organic_or_paid)
+        self.util.write_to_gsheet(gsheet_name,tab_name,data_recency,sheet_prefix=organic_or_paid)
 
         #create a tag string to identify a channel, a concatenation of all the column values specified in 'cols_to_group'
         def concat_cols(x):
@@ -136,28 +125,20 @@ class QualityAssessments:
 
         return error_message
     
-    def boosted_function_qa(self,paid_df, organic_df, impressions_threshold = 100000):
-        """Takes in organic data and paid data and reports how many of the posts labelled 
-            as boosted in the paid data are present in the organic data
+    def boosted_function_qa(self,paid_df, organic_df, gsheet_name,tab_name='OrganicWithBigImpressions',impressions_threshold = 100000):
+        """Takes in organic data and paid data and reports how many are mislabelled as boosted
             
-            Parameters
-            ----------------
-            paid_df : DataFrame
-                Daily ad spend, boosted posts identified in the 'workstream' column
-            organic_df : DataFrame
-                Organic Post performance Data 
-            impressions_threshold : 
-                the number of impressions above which it is unlikely the post is purely organic
+        Args:
+            paid_df (pandas.DataFrame): Daily ad spend, boosted posts identified in the 'workstream' column
+            organic_df (pandas.DataFrame): Organic Post performance Data 
+            impressions_threshold (pandas.DataFrame): The number of impressions above which it is unlikely the post is purely organic
             
-            Returns
-            ---------------
-            error_message : str
-                String detailing what the error is so that it can be passed to a notification 
-                service like slack"""
+        Returns:
+            error_message (str): A string detailing what the error is so that it can be passed to a notification service like slack"""
         error_message = ''
         organic_df['Error'] = False
 
-        # count number of unique paid posts with workstream organic minus oranic posts labelled as boosted
+        # count number of unique paid posts wi  th workstream organic minus oranic posts labelled as boosted
         # number of boosted post from paid naming convention Tags
         paid_boosted_count = len(paid_df[paid_df['workstream'] == 'boosted']['post_id'].unique())
 
@@ -180,7 +161,7 @@ class QualityAssessments:
             error_message = error_message + '  ' + \
                 f'There are {len(misslabelled_og_rows.index.values)} Pure Organic posts with over {impressions_threshold} impressions or video views\n'
             logger.warning(error_message)
-            self.util.write_to_gsheet(self.main_error_tracker_name,'OrganicWithBigImpressions', misslabelled_og_rows)
+            self.util.write_to_gsheet(gsheet_name,tab_name, misslabelled_og_rows)
         # return TT or IG values with empty message field
         # for the date function exlude the dates we paused for the queen
 
@@ -189,33 +170,23 @@ class QualityAssessments:
     
     def comparison_with_previous_data(self,df,name_of_df,cols_to_check=['impressions','likes'],perc_increase_threshold=20,
                                    perc_decrease_threshold=0.5, check_cols_set=True,raise_exceptions=False):
-        '''This function stores the high level sums from the previous run of the script
-            and if they have reduced or increased too sharply an error is raised
-        Parameters
-        --------------------
-            df : DataFrame
-                Input dataframe that the historic checks are going to be performed on
-            name_of_df : str
-                Name of the dataframe, this will be used to name a file to save for future comparison
-            cols_to_check : list, str
-                A list of strings that detail the columns to be totaled which will then be compared with previous data
-            perc_increase_threshold : number
-                A number between 0 and 100, the percentage increase threshold above which it is deemed that the totals 
-                have raised too rapidly and an error has occured
-            perc_decrease_threshold : number
-                A number between 0 and 100, the percentage decrease threshold below which it is deemed that the totals
-                decreased and an error has occured
-            check_cols_set : bool
-                If true, store the set of columns present for comparison to see if any new columns have been added or
-                removed next time, in which case it is deemed an error has occured
-            raise_exceptions : bool
-                If true, then raise an exception if an error has occured instead of just returning an error message
+        """ This function stores the high level sums for a datatable from the previous run of the script
+        and if they have reduced or increased too sharply an error is raised.
+        Args:
+            df (pd.DataFrame): Input dataframe that the historic checks are going to be performed on.
+            name_of_df (str): Name of the dataframe, this will be used to name a file to save for future comparison.
+            cols_to_check (List[str]): A list of strings that detail the columns to be totaled which will then 
+                be compared with previous data.
+            perc_increase_threshold (float): A number between 0 and 100, the percentage increase threshold above 
+                which it is deemed that the totals have raised too rapidly and an error has occured.
+            perc_decrease_threshold (float): A number between 0 and 100, the percentage decrease threshold below 
+                which it is deemed that the totals decreased and an error has occured.
+            check_cols_set (bool): If true, store the set of columns present for comparison to see if any new 
+                columns have been added or removed next time, in which case it is deemed an error has occured.
+            raise_exceptions (bool): If true, then raise an exception if an error has occured instead of just returning an error message.
 
-        Returns
-        -------------------
-            error_message : str
-                String detailing what the error is so that it can be passed to a notification 
-                service like slack'''
+        Returns:
+            error_message (str): String detailing what the error is so that it can be passed to a notification service like slack."""
         
         error_message, error_occured = '', False
         new_dict = {}
@@ -263,26 +234,17 @@ class QualityAssessments:
         
         return error_message
     
-    def duplicates_qa(self,df,df_name,subset=None, drop_duplicates=True):
-        """Checks for duplicates and optionally drops duplicates in a dataframe
-        Parameters
-        -----------------
-            df : DataFrame
-                The Dataframe to be checked for duplicates
-            df_name : str
-                The name of the dataframe to be used for logging purposes
-            subset : list, str
-                Only consider certain columns for identifying duplicates, by default use all of the columns
-            drop_duplicates : bool
-                If true remove duplicate values
+    def duplicates_qa(self, df: pd.DataFrame, df_name: str,subset= None, drop_duplicates: bool = True):
+        """Checks for duplicates and optionally drops duplicates in a dataframe.
         
-        Returns
-        ----------------
-            df : DataFrame
-                Returns original dataframe without duplicates if 'drop_duplicates' = True
-            error_message : str
-                String detailing what the error is so that it can be passed to a notification 
-                service like slack"""
+        Args:
+            df (pd.DataFrame): The Dataframe to be checked for duplicates
+            df_name (str): The name of the dataframe to be used for logging purposes
+            subset (Optional[Union[list, str]], optional): Only consider certain columns for identifying duplicates, by default use all of the columns
+            drop_duplicates (bool, optional): If true remove duplicate values
+            
+        Returns:
+            df (pd.DataFrame): Returns original dataframe without duplicates if 'drop_duplicates' = True"""
 
         num_duplicates = df.duplicated(subset=subset).sum()
         if num_duplicates >= 0:
@@ -290,24 +252,22 @@ class QualityAssessments:
             if drop_duplicates:
                 df.drop_duplicates(subset=subset, inplace=True)
         return df
+
     
-    def check_impressions_no_engagements(self,df, raise_exceptions=False):
-        """Function to check if a row item has engagements but no impressions and no video views
-            This shouldn't happen andis indicative of an error with Tracer but can be valid as 
-            some platforms count viral engagements differently
-            Parameters
-            ------------------
-            df : DataFrame
-                Input dataframe of advertising data with columns 'impressions' or 'video_views'
-            raise_exceptions : bool
-                Boolean flag if set to true will raise an exception if an offending row item is discovered
-            
-            Returns
-            ------------------
-            error_message : str
-                String detailing what the error is so that it can be passed to a notification 
-                service like slack
-                """
+    def check_impressions_no_engagements(self,df,gsheet_name,tab_name='NoImpressionsButEngagements',raise_exceptions=False):
+        """Function to check if a row item has engagements but no impressions and no video views. 
+        This shouldn't happen and is indicative of an error with Tracer but can be valid as some 
+        platforms count viral engagements differently.
+
+        Args:
+            df (DataFrame): Input dataframe of advertising data with columns 'impressions' or 'video_views'
+            gsheet_name (str): name of the google sheet
+            tab_name (str, optional): name of the tab. Defaults to 'NoImpressionsButEngagements'.
+            raise_exceptions (bool, optional): Boolean flag if set to true will raise an exception if an offending row item is discovered. Defaults to False.
+
+        Returns:
+            error_message (str): String detailing what the error is so that it can be passed to a notification service like slack."""
+
         paid_or_organic = self.util.identify_paid_or_organic(df)
         # https://docs.google.com/spreadsheets/d/1refbPLje6B48qvSRNXrK3U_OAfzjzeuTrzgfqq9O-yw/edit#gid=0
         df['date'] = pd.to_datetime(df['date'])
@@ -336,7 +296,7 @@ class QualityAssessments:
             df.loc[no_impressions_but_engage_rows, 'Error'] = True
             # error_message = error_message + " " + "Split By platform"
             # error_message = error_message + " " + pd.crosstab(erroneous_df['platform'],erroneous_df['media_type'],margins=True,dropna=False)
-            self.util.write_to_gsheet("Indeed Data Error Tracking Tracer", 'NoImpressionsButEngagements', df.loc[no_impressions_but_engage_rows],
+            self.util.write_to_gsheet(gsheet_name, tab_name, df.loc[no_impressions_but_engage_rows],
                             sheet_prefix=paid_or_organic)
             exception_occurred = True
 
@@ -346,31 +306,36 @@ class QualityAssessments:
         
         return error_message
 
-    def naming_convention_checker(self, df,naming_convention,campaignname_dict=None, adgroupname_dict=None, 
+    def naming_convention_checker(self, df,gsheet_name,naming_convention,campaignname_dict=None, adgroupname_dict=None, 
                                     adname_dict=None,campaign_col='campaign_name',adgroup_col='group_name',adname_col='name',
                                     start_char='_',middle_char=':',end_char='_'):
-        """Takes in Paid Data with cleaned columns and the naming convention google sheet and
-        for each unique campaign_name, adgroup_name and adname will show whether the key is missing or whether
-        the values is incorrect
-        
-        Parameters
-        ---------------------- 
-        df : DataFrame 
-            The Dataframe containing paid df with the campaign name, 
-            ad group name and ad name column included
-        naming_convention :  dataframe
-            The dataframe extracted froom a google sheet which contains all the accepted
-            tags and each tag's acceptable values
-        campaignname_dict : dictionary
-            A dictionary of the tags to be checked, column label as the key, shortcode as the value
-        adgroupname_dict : dictionary
-            A dictionary of the tags to be checked, column label as the key, shortcode as the value
-        adname_dict = dictionary
-            A dictionary of the tags to be checked, column label as the key, shortcode as the value
-        
-        Returns
-        ---------------------
-        Writes to google sheet a table with the index being a unique instance of the campaign"""
+        """Checks for naming convention errors in a given DataFrame and outputs the errors to a Google Sheet.
+
+        The function takes in a DataFrame containing paid data with columns for campaign name, ad group name, and ad name, 
+        as well as a DataFrame containing the accepted tags and values for each level of naming (campaign, ad group, ad name). 
+        Dictionarys for each level fo checking are required this specifies what tags should be checked for, e.g. should a check
+        for the correct values in 'platform' be performed. If a dictionary for a certain level is not provided that level will
+        not be checked. The function will output a table in a Google Sheet showing all errors in naming conventions, 
+        with a tab for each level of errors.
+
+        The Google Sheet should be set up with a tab name for each level of errors to check for, 
+        i.e. 'CampaignNameErrors', 'AdGroupNameErrors', 'AdNameErrors'.
+
+        Args:
+            df (DataFrame): The DataFrame containing paid data with campaign name, ad group name, and ad name columns.
+            naming_convention (DataFrame): The DataFrame containing the accepted tags and values for each level of naming.
+            campaignname_dict (dict, optional): A dictionary of the tags to be checked for the campaign name, with the column label as the key and the shortcode as the value.
+            adgroupname_dict (dict, optional): A dictionary of the tags to be checked for the ad group name, with the column label as the key and the shortcode as the value.
+            adname_dict (dict, optional): A dictionary of the tags to be checked for the ad name, with the column label as the key and the shortcode as the value.
+            campaign_col (str, optional): The column in the input DataFrame that corresponds to the campaign name. Defaults to 'campaign_name'.
+            adgroup_col (str, optional): The column in the input DataFrame that corresponds to the ad group name. Defaults to 'group_name'.
+            adname_col (str, optional): The column in the input DataFrame that corresponds to the ad name. Defaults to 'name'.
+            start_char (str, optional): The starting character for the tag in the naming convention. Defaults to '_'.
+            middle_char (str, optional): The character that separates the tag from the value in the naming convention. Defaults to ':'.
+            end_char (str, optional): The ending character for the tag in the naming convention. Defaults to '_'.
+
+        Returns:
+            Writes to a Google Sheet a table with the index being a unique instance of the campaign."""
         
         conv_level_tuple = ((campaignname_dict,campaign_col,'CampaignNameErrors'),(adgroupname_dict,adgroup_col,'AdGroupNameErrors'),
                             (adname_dict,adname_col,'AdNameErrors'))
@@ -407,7 +372,6 @@ class QualityAssessments:
                 #For each label and tag in the required set 
                 if label in key_values_conv_cols:
                     acceptable_values= remove_empties_from_list(naming_convention[label+' Key'].str.upper().unique().tolist())
-                    output_df[f'{label} ("{tag}")'] = output_df[level[1]].apply(lambda x: 
-                                                                return_value(x,tag,acceptable_values))
-            self.util.write_to_gsheet(self.naming_conv_tracker_name, level[2],output_df)
+                    output_df[f'{label} ("{tag}")'] = output_df[level[1]].apply(lambda x: return_value(x,tag,acceptable_values))
+            self.util.write_to_gsheet(workbook_name = gsheet_name,sheet_name= level[2],df = output_df)
     
